@@ -1,26 +1,15 @@
-"""Tests for various magic functions specific to the terminal frontend.
-
-Needs to be run by nose (to make ipython session available).
-"""
-from __future__ import absolute_import
+"""Tests for various magic functions specific to the terminal frontend."""
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
 import sys
-from StringIO import StringIO
+from io import StringIO
 from unittest import TestCase
 
-import nose.tools as nt
-
 from IPython.testing import tools as tt
 
-#-----------------------------------------------------------------------------
-# Globals
-#-----------------------------------------------------------------------------
-ip = get_ipython()
-
 #-----------------------------------------------------------------------------
 # Test functions begin
 #-----------------------------------------------------------------------------
@@ -32,9 +21,6 @@ def check_cpaste(code, should_fail=False):
     ip.user_ns['code_ran'] = False
 
     src = StringIO()
-    if not hasattr(src, 'encoding'):
-        # IPython expects stdin to have an encoding attribute
-        src.encoding = None
     src.write(code)
     src.write('\n--\n')
     src.seek(0)
@@ -52,8 +38,6 @@ def check_cpaste(code, should_fail=False):
     finally:
         sys.stdin = stdin_save
 
-PY31 = sys.version_info[:2] == (3,1)
-
 def test_cpaste():
     """Test cpaste magic"""
 
@@ -72,13 +56,8 @@ def test_cpaste():
                       ],
 
              'fail': ["1 + runf()",
+                      "++ runf()",
              ]}
-    
-    # I don't know why this is failing specifically on Python 3.1. I've
-    # checked it manually interactively, but we don't care enough about 3.1
-    # to spend time fiddling with the tests, so we just skip it.
-    if not PY31:
-        tests['fail'].append("++ runf()")
 
     ip.user_ns['runf'] = runf
 
@@ -106,16 +85,16 @@ class PasteTestCase(TestCase):
         ip.hooks.clipboard_get = self.original_clip
        
     def test_paste(self):
-        ip.user_ns.pop('x', None)
-        self.paste('x = 1')
-        nt.assert_equal(ip.user_ns['x'], 1)
-        ip.user_ns.pop('x')
+        ip.user_ns.pop("x", None)
+        self.paste("x = 1")
+        self.assertEqual(ip.user_ns["x"], 1)
+        ip.user_ns.pop("x")
 
     def test_paste_pyprompt(self):
-        ip.user_ns.pop('x', None)
-        self.paste('>>> x=2')
-        nt.assert_equal(ip.user_ns['x'], 2)
-        ip.user_ns.pop('x')
+        ip.user_ns.pop("x", None)
+        self.paste(">>> x=2")
+        self.assertEqual(ip.user_ns["x"], 2)
+        ip.user_ns.pop("x")
 
     def test_paste_py_multi(self):
         self.paste("""
@@ -124,35 +103,38 @@ class PasteTestCase(TestCase):
         >>> for i in x:
         ...     y.append(i**2)
         ... 
-        """)
-        nt.assert_equal(ip.user_ns['x'], [1,2,3])
-        nt.assert_equal(ip.user_ns['y'], [1,4,9])
+        """
+        )
+        self.assertEqual(ip.user_ns["x"], [1, 2, 3])
+        self.assertEqual(ip.user_ns["y"], [1, 4, 9])
 
     def test_paste_py_multi_r(self):
         "Now, test that self.paste -r works"
         self.test_paste_py_multi()
-        nt.assert_equal(ip.user_ns.pop('x'), [1,2,3])
-        nt.assert_equal(ip.user_ns.pop('y'), [1,4,9])
-        nt.assert_false('x' in ip.user_ns)
-        ip.magic('paste -r')
-        nt.assert_equal(ip.user_ns['x'], [1,2,3])
-        nt.assert_equal(ip.user_ns['y'], [1,4,9])
+        self.assertEqual(ip.user_ns.pop("x"), [1, 2, 3])
+        self.assertEqual(ip.user_ns.pop("y"), [1, 4, 9])
+        self.assertFalse("x" in ip.user_ns)
+        ip.magic("paste -r")
+        self.assertEqual(ip.user_ns["x"], [1, 2, 3])
+        self.assertEqual(ip.user_ns["y"], [1, 4, 9])
 
     def test_paste_email(self):
         "Test pasting of email-quoted contents"
         self.paste("""\
         >> def foo(x):
         >>     return x + 1
-        >> xx = foo(1.1)""")
-        nt.assert_equal(ip.user_ns['xx'], 2.1)
+        >> xx = foo(1.1)"""
+        )
+        self.assertEqual(ip.user_ns["xx"], 2.1)
 
     def test_paste_email2(self):
         "Email again; some programs add a space also at each quoting level"
         self.paste("""\
         > > def foo(x):
         > >     return x + 1
-        > > yy = foo(2.1)     """)
-        nt.assert_equal(ip.user_ns['yy'], 3.1)
+        > > yy = foo(2.1)     """
+        )
+        self.assertEqual(ip.user_ns["yy"], 3.1)
 
     def test_paste_email_py(self):
         "Email quoting of interactive input"
@@ -160,14 +142,15 @@ class PasteTestCase(TestCase):
         >> >>> def f(x):
         >> ...   return x+1
         >> ... 
-        >> >>> zz = f(2.5)      """)
-        nt.assert_equal(ip.user_ns['zz'], 3.5)
+        >> >>> zz = f(2.5)      """
+        )
+        self.assertEqual(ip.user_ns["zz"], 3.5)
 
     def test_paste_echo(self):
         "Also test self.paste echoing, by temporarily faking the writer"
         w = StringIO()
-        writer = ip.write
-        ip.write = w.write
+        old_write = sys.stdout.write
+        sys.stdout.write = w.write
         code = """
         a = 100
         b = 200"""
@@ -175,10 +158,10 @@ class PasteTestCase(TestCase):
             self.paste(code,'')
             out = w.getvalue()
         finally:
-            ip.write = writer
-        nt.assert_equal(ip.user_ns['a'], 100)
-        nt.assert_equal(ip.user_ns['b'], 200)
-        nt.assert_equal(out, code+"\n## -- End pasted text --\n")
+            sys.stdout.write = old_write
+        self.assertEqual(ip.user_ns["a"], 100)
+        self.assertEqual(ip.user_ns["b"], 200)
+        assert out == code + "\n## -- End pasted text --\n"
 
     def test_paste_leading_commas(self):
         "Test multiline strings with leading commas"
@@ -187,10 +170,9 @@ class PasteTestCase(TestCase):
 a = """
 ,1,2,3
 """'''
-        ip.user_ns.pop('foo', None)
-        tm.store_or_execute(s, 'foo')
-        nt.assert_in('foo', ip.user_ns)
-
+        ip.user_ns.pop("foo", None)
+        tm.store_or_execute(s, "foo")
+        self.assertIn("foo", ip.user_ns)
 
     def test_paste_trailing_question(self):
         "Test pasting sources with trailing question marks"
@@ -202,4 +184,4 @@ def funcfoo():
 '''
         ip.user_ns.pop('funcfoo', None)
         self.paste(s)
-        nt.assert_equal(ip.user_ns['funcfoo'](), 'fooresult')
+        self.assertEqual(ip.user_ns["funcfoo"](), "fooresult")
