@@ -4,59 +4,46 @@
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-import errno
 import os
 import shutil
 import sys
 import tempfile
-import warnings
+import unittest
 from contextlib import contextmanager
+from unittest.mock import patch
+from os.path import join, abspath
+from importlib import reload
 
-try:  # Python 3.3+
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch
-
-from os.path import join, abspath, split
-
-from nose import SkipTest
-import nose.tools as nt
-
-from nose import with_setup
+import pytest
 
 import IPython
 from IPython import paths
 from IPython.testing import decorators as dec
-from IPython.testing.decorators import (skip_if_not_win32, skip_win32,
-                                        onlyif_unicode_paths,)
-from IPython.testing.tools import make_tempfile, AssertPrints
+from IPython.testing.decorators import (
+    skip_if_not_win32,
+    skip_win32,
+    onlyif_unicode_paths,
+)
+from IPython.testing.tools import make_tempfile
 from IPython.utils import path
-from IPython.utils import py3compat
 from IPython.utils.tempdir import TemporaryDirectory
 
+
 # Platform-dependent imports
 try:
-    import winreg as wreg  # Py 3
+    import winreg as wreg
 except ImportError:
+    #Fake _winreg module on non-windows platforms
+    import types
+    wr_name = "winreg"
+    sys.modules[wr_name] = types.ModuleType(wr_name)
     try:
-        import _winreg as wreg  # Py 2
+        import winreg as wreg
     except ImportError:
-        #Fake _winreg module on none windows platforms
-        import types
-        wr_name = "winreg" if py3compat.PY3 else "_winreg"
-        sys.modules[wr_name] = types.ModuleType(wr_name)
-        try:
-            import winreg as wreg
-        except ImportError:
-            import _winreg as wreg
+        import _winreg as wreg
         #Add entries that needs to be stubbed by the testing code
         (wreg.OpenKey, wreg.QueryValueEx,) = (None, None)
 
-try:
-    reload
-except NameError:   # Python 3
-    from imp import reload
-
 #-----------------------------------------------------------------------------
 # Globals
 #-----------------------------------------------------------------------------
@@ -67,20 +54,20 @@ HOME_TEST_DIR = join(TMP_TEST_DIR, "home_test_dir")
 # Setup/teardown functions/decorators
 #
 
-def setup():
+def setup_module():
     """Setup testenvironment for the module:
 
-            - Adds dummy home dir tree
+    - Adds dummy home dir tree
     """
     # Do not mask exceptions here.  In particular, catching WindowsError is a
     # problem because that exception is only defined on Windows...
     os.makedirs(os.path.join(HOME_TEST_DIR, 'ipython'))
 
 
-def teardown():
+def teardown_module():
     """Teardown testenvironment for the module:
 
-            - Remove dummy home dir tree
+    - Remove dummy home dir tree
     """
     # Note: we remove the parent test dir, which is the root of all test
     # subdirs we may have created.  Use shutil instead of os.removedirs, so
@@ -112,8 +99,17 @@ def teardown_environment():
     if hasattr(sys, 'frozen'):
         del sys.frozen
 
+
 # Build decorator that uses the setup_environment/setup_environment
-with_environment = with_setup(setup_environment, teardown_environment)
+@pytest.fixture
+def environment():
+    setup_environment()
+    yield
+    teardown_environment()
+
+
+with_environment = pytest.mark.usefixtures("environment")
+
 
 @skip_if_not_win32
 @with_environment
@@ -127,7 +123,7 @@ def test_get_home_dir_1():
     IPython.__file__ = abspath(join(HOME_TEST_DIR, "Lib/IPython/__init__.py"))
 
     home_dir = path.get_home_dir()
-    nt.assert_equal(home_dir, unfrozen)
+    assert home_dir == unfrozen
 
 
 @skip_if_not_win32
@@ -141,16 +137,17 @@ def test_get_home_dir_2():
     IPython.__file__ = abspath(join(HOME_TEST_DIR, "Library.zip/IPython/__init__.py")).lower()
 
     home_dir = path.get_home_dir(True)
-    nt.assert_equal(home_dir, unfrozen)
+    assert home_dir == unfrozen
 
 
+@skip_win32
 @with_environment
 def test_get_home_dir_3():
     """get_home_dir() uses $HOME if set"""
     env["HOME"] = HOME_TEST_DIR
     home_dir = path.get_home_dir(True)
     # get_home_dir expands symlinks
-    nt.assert_equal(home_dir, os.path.realpath(env["HOME"]))
+    assert home_dir == os.path.realpath(env["HOME"])
 
 
 @with_environment
@@ -161,13 +158,14 @@ def test_get_home_dir_4():
     # this should still succeed, but we don't care what the answer is
     home = path.get_home_dir(False)
 
+@skip_win32
 @with_environment
 def test_get_home_dir_5():
     """raise HomeDirError if $HOME is specified, but not a writable dir"""
     env['HOME'] = abspath(HOME_TEST_DIR+'garbage')
     # set os.name = posix, to prevent My Documents fallback on Windows
     os.name = 'posix'
-    nt.assert_raises(path.HomeDirError, path.get_home_dir, True)
+    pytest.raises(path.HomeDirError, path.get_home_dir, True)
 
 # Should we stub wreg fully so we can run the test on all platforms?
 @skip_if_not_win32
@@ -183,13 +181,17 @@ def test_get_home_dir_8():
         env.pop(key, None)
 
     class key:
+        def __enter__(self):
+            pass
         def Close(self):
             pass
+        def __exit__(*args, **kwargs):
+            pass
 
     with patch.object(wreg, 'OpenKey', return_value=key()), \
          patch.object(wreg, 'QueryValueEx', return_value=[abspath(HOME_TEST_DIR)]):
         home_dir = path.get_home_dir()
-    nt.assert_equal(home_dir, abspath(HOME_TEST_DIR))
+    assert home_dir == abspath(HOME_TEST_DIR)
 
 @with_environment
 def test_get_xdg_dir_0():
@@ -203,12 +205,12 @@ def test_get_xdg_dir_0():
     env.pop('IPYTHONDIR', None)
     env.pop('XDG_CONFIG_HOME', None)
 
-    nt.assert_equal(path.get_xdg_dir(), os.path.join('somewhere', '.config'))
+    assert path.get_xdg_dir() == os.path.join("somewhere", ".config")
 
 
 @with_environment
 def test_get_xdg_dir_1():
-    """test_get_xdg_dir_1, check nonexistant xdg_dir"""
+    """test_get_xdg_dir_1, check nonexistent xdg_dir"""
     reload(path)
     path.get_home_dir = lambda : HOME_TEST_DIR
     os.name = "posix"
@@ -216,7 +218,7 @@ def test_get_xdg_dir_1():
     env.pop('IPYTHON_DIR', None)
     env.pop('IPYTHONDIR', None)
     env.pop('XDG_CONFIG_HOME', None)
-    nt.assert_equal(path.get_xdg_dir(), None)
+    assert path.get_xdg_dir() is None
 
 @with_environment
 def test_get_xdg_dir_2():
@@ -232,23 +234,22 @@ def test_get_xdg_dir_2():
     if not os.path.exists(cfgdir):
         os.makedirs(cfgdir)
 
-    nt.assert_equal(path.get_xdg_dir(), cfgdir)
+    assert path.get_xdg_dir() == cfgdir
 
 @with_environment
 def test_get_xdg_dir_3():
-    """test_get_xdg_dir_3, check xdg_dir not used on OS X"""
+    """test_get_xdg_dir_3, check xdg_dir not used on non-posix systems"""
     reload(path)
     path.get_home_dir = lambda : HOME_TEST_DIR
-    os.name = "posix"
-    sys.platform = "darwin"
+    os.name = "nt"
+    sys.platform = "win32"
     env.pop('IPYTHON_DIR', None)
     env.pop('IPYTHONDIR', None)
     env.pop('XDG_CONFIG_HOME', None)
     cfgdir=os.path.join(path.get_home_dir(), '.config')
-    if not os.path.exists(cfgdir):
-        os.makedirs(cfgdir)
+    os.makedirs(cfgdir, exist_ok=True)
 
-    nt.assert_equal(path.get_xdg_dir(), None)
+    assert path.get_xdg_dir() is None
 
 def test_filefind():
     """Various tests for filefind"""
@@ -263,78 +264,82 @@ def test_filefind():
 def test_get_long_path_name_win32():
     with TemporaryDirectory() as tmpdir:
 
-        # Make a long path. Expands the path of tmpdir prematurely as it may already have a long 
+        # Make a long path. Expands the path of tmpdir prematurely as it may already have a long
         # path component, so ensure we include the long form of it
-        long_path = os.path.join(path.get_long_path_name(tmpdir), u'this is my long path name')
+        long_path = os.path.join(path.get_long_path_name(tmpdir), 'this is my long path name')
         os.makedirs(long_path)
 
         # Test to see if the short path evaluates correctly.
-        short_path = os.path.join(tmpdir, u'THISIS~1')
+        short_path = os.path.join(tmpdir, 'THISIS~1')
         evaluated_path = path.get_long_path_name(short_path)
-        nt.assert_equal(evaluated_path.lower(), long_path.lower())
+        assert evaluated_path.lower() == long_path.lower()
 
 
 @dec.skip_win32
 def test_get_long_path_name():
-    p = path.get_long_path_name('/usr/local')
-    nt.assert_equal(p,'/usr/local')
+    p = path.get_long_path_name("/usr/local")
+    assert p == "/usr/local"
 
-@dec.skip_win32 # can't create not-user-writable dir on win
-@with_environment
-def test_not_writable_ipdir():
-    tmpdir = tempfile.mkdtemp()
-    os.name = "posix"
-    env.pop('IPYTHON_DIR', None)
-    env.pop('IPYTHONDIR', None)
-    env.pop('XDG_CONFIG_HOME', None)
-    env['HOME'] = tmpdir
-    ipdir = os.path.join(tmpdir, '.ipython')
-    os.mkdir(ipdir, 0o555)
-    try:
-        open(os.path.join(ipdir, "_foo_"), 'w').close()
-    except IOError:
-        pass
-    else:
-        # I can still write to an unwritable dir,
-        # assume I'm root and skip the test
-        raise SkipTest("I can't create directories that I can't write to")
-    with AssertPrints('is not a writable location', channel='stderr'):
-        ipdir = paths.get_ipython_dir()
-    env.pop('IPYTHON_DIR', None)
+
+class TestRaiseDeprecation(unittest.TestCase):
+
+    @dec.skip_win32 # can't create not-user-writable dir on win
+    @with_environment
+    def test_not_writable_ipdir(self):
+        tmpdir = tempfile.mkdtemp()
+        os.name = "posix"
+        env.pop('IPYTHON_DIR', None)
+        env.pop('IPYTHONDIR', None)
+        env.pop('XDG_CONFIG_HOME', None)
+        env['HOME'] = tmpdir
+        ipdir = os.path.join(tmpdir, '.ipython')
+        os.mkdir(ipdir, 0o555)
+        try:
+            open(os.path.join(ipdir, "_foo_"), 'w').close()
+        except IOError:
+            pass
+        else:
+            # I can still write to an unwritable dir,
+            # assume I'm root and skip the test
+            pytest.skip("I can't create directories that I can't write to")
+
+        with self.assertWarnsRegex(UserWarning, 'is not a writable location'):
+            ipdir = paths.get_ipython_dir()
+        env.pop('IPYTHON_DIR', None)
 
 @with_environment
 def test_get_py_filename():
     os.chdir(TMP_TEST_DIR)
-    with make_tempfile('foo.py'):
-        nt.assert_equal(path.get_py_filename('foo.py'), 'foo.py')
-        nt.assert_equal(path.get_py_filename('foo'), 'foo.py')
-    with make_tempfile('foo'):
-        nt.assert_equal(path.get_py_filename('foo'), 'foo')
-        nt.assert_raises(IOError, path.get_py_filename, 'foo.py')
-    nt.assert_raises(IOError, path.get_py_filename, 'foo')
-    nt.assert_raises(IOError, path.get_py_filename, 'foo.py')
-    true_fn = 'foo with spaces.py'
+    with make_tempfile("foo.py"):
+        assert path.get_py_filename("foo.py") == "foo.py"
+        assert path.get_py_filename("foo") == "foo.py"
+    with make_tempfile("foo"):
+        assert path.get_py_filename("foo") == "foo"
+        pytest.raises(IOError, path.get_py_filename, "foo.py")
+    pytest.raises(IOError, path.get_py_filename, "foo")
+    pytest.raises(IOError, path.get_py_filename, "foo.py")
+    true_fn = "foo with spaces.py"
     with make_tempfile(true_fn):
-        nt.assert_equal(path.get_py_filename('foo with spaces'), true_fn)
-        nt.assert_equal(path.get_py_filename('foo with spaces.py'), true_fn)
-        nt.assert_raises(IOError, path.get_py_filename, '"foo with spaces.py"')
-        nt.assert_raises(IOError, path.get_py_filename, "'foo with spaces.py'")
+        assert path.get_py_filename("foo with spaces") == true_fn
+        assert path.get_py_filename("foo with spaces.py") == true_fn
+        pytest.raises(IOError, path.get_py_filename, '"foo with spaces.py"')
+        pytest.raises(IOError, path.get_py_filename, "'foo with spaces.py'")
 
 @onlyif_unicode_paths
 def test_unicode_in_filename():
     """When a file doesn't exist, the exception raised should be safe to call
     str() on - i.e. in Python 2 it must only have ASCII characters.
-    
+
     https://github.com/ipython/ipython/issues/875
     """
     try:
         # these calls should not throw unicode encode exceptions
-        path.get_py_filename(u'fooéè.py',  force_win32=False)
+        path.get_py_filename('fooéè.py')
     except IOError as ex:
         str(ex)
 
 
-class TestShellGlob(object):
+class TestShellGlob(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
@@ -356,7 +361,7 @@ class TestShellGlob(object):
     @classmethod
     @contextmanager
     def in_tempdir(cls):
-        save = py3compat.getcwd()
+        save = os.getcwd()
         try:
             os.chdir(cls.tempdir.name)
             yield
@@ -366,8 +371,7 @@ class TestShellGlob(object):
     def check_match(self, patterns, matches):
         with self.in_tempdir():
             # glob returns unordered list. that's why sorted is required.
-            nt.assert_equals(sorted(path.shellglob(patterns)),
-                             sorted(matches))
+            assert sorted(path.shellglob(patterns)) == sorted(matches)
 
     def common_cases(self):
         return [
@@ -388,7 +392,7 @@ class TestShellGlob(object):
                 ([r'a\*', 'a*'], ['a*'] + self.filenames_start_with_a),
                 ([r'a\[012]'], ['a[012]']),
                 ]:
-            yield (self.check_match, patterns, matches)
+            self.check_match(patterns, matches)
 
     @skip_if_not_win32
     def test_match_windows(self):
@@ -399,30 +403,31 @@ class TestShellGlob(object):
                 ([r'a\*', 'a*'], [r'a\*'] + self.filenames_start_with_a),
                 ([r'a\[012]'], [r'a\[012]']),
                 ]:
-            yield (self.check_match, patterns, matches)
+            self.check_match(patterns, matches)
 
 
+# TODO : pytest.mark.parametrise once nose is gone.
 def test_unescape_glob():
-    nt.assert_equals(path.unescape_glob(r'\*\[\!\]\?'), '*[!]?')
-    nt.assert_equals(path.unescape_glob(r'\\*'), r'\*')
-    nt.assert_equals(path.unescape_glob(r'\\\*'), r'\*')
-    nt.assert_equals(path.unescape_glob(r'\\a'), r'\a')
-    nt.assert_equals(path.unescape_glob(r'\a'), r'\a')
+    assert path.unescape_glob(r"\*\[\!\]\?") == "*[!]?"
+    assert path.unescape_glob(r"\\*") == r"\*"
+    assert path.unescape_glob(r"\\\*") == r"\*"
+    assert path.unescape_glob(r"\\a") == r"\a"
+    assert path.unescape_glob(r"\a") == r"\a"
 
 
 @onlyif_unicode_paths
 def test_ensure_dir_exists():
     with TemporaryDirectory() as td:
-        d = os.path.join(td, u'∂ir')
+        d = os.path.join(td, '∂ir')
         path.ensure_dir_exists(d) # create it
         assert os.path.isdir(d)
         path.ensure_dir_exists(d) # no-op
-        f = os.path.join(td, u'ƒile')
+        f = os.path.join(td, 'ƒile')
         open(f, 'w').close() # touch
-        with nt.assert_raises(IOError):
+        with pytest.raises(IOError):
             path.ensure_dir_exists(f)
 
-class TestLinkOrCopy(object):
+class TestLinkOrCopy(unittest.TestCase):
     def setUp(self):
         self.tempdir = TemporaryDirectory()
         self.src = self.dst("src")
@@ -436,17 +441,19 @@ class TestLinkOrCopy(object):
         return os.path.join(self.tempdir.name, *args)
 
     def assert_inode_not_equal(self, a, b):
-        nt.assert_not_equals(os.stat(a).st_ino, os.stat(b).st_ino,
-                             "%r and %r do reference the same indoes" %(a, b))
+        assert (
+            os.stat(a).st_ino != os.stat(b).st_ino
+        ), "%r and %r do reference the same indoes" % (a, b)
 
     def assert_inode_equal(self, a, b):
-        nt.assert_equals(os.stat(a).st_ino, os.stat(b).st_ino,
-                         "%r and %r do not reference the same indoes" %(a, b))
+        assert (
+            os.stat(a).st_ino == os.stat(b).st_ino
+        ), "%r and %r do not reference the same indoes" % (a, b)
 
     def assert_content_equal(self, a, b):
         with open(a) as a_f:
             with open(b) as b_f:
-                nt.assert_equals(a_f.read(), b_f.read())
+                assert a_f.read() == b_f.read()
 
     @skip_win32
     def test_link_successful(self):
@@ -494,4 +501,4 @@ class TestLinkOrCopy(object):
         path.link_or_copy(self.src, dst)
         path.link_or_copy(self.src, dst)
         self.assert_inode_equal(self.src, dst)
-        nt.assert_equal(sorted(os.listdir(self.tempdir.name)), ['src', 'target'])
+        assert sorted(os.listdir(self.tempdir.name)) == ["src", "target"]
